const { MongoClient } = require("mongodb");
const crypto = require("crypto");

const uri = "mongodb://localhost:27017/";
const client = new MongoClient(uri);

async function connectToMongo() {
  try {
    await client.connect({
      useNewUrlParser: true,
      useUnifiedTopology: true,
      autoEncryption: {
        kmsProviders: {
          local: {
            key: crypto.randomBytes(96)
          }
        },
        keyVaultNamespace: "myDatabase" + ".__keystore"
      }
    });
    console.log("Connected to MongoDB");
  } catch (err) {
    console.error("Error connecting to MongoDB:", err);
    throw err;
  }
}

async function insert() {
  try {
    const db = client.db("myDatabase");
    const collection = db.collection("myCollection");
    await collection.insertOne({
      firstName: "Peter",
      lastName: "Parker",
      ssn: "901-01-1234"
    });
    console.log("Data inserted successfully and in encrypted form");
  } catch (err) {
    console.error("Error inserting encrypted data:", err);
    throw err;
  }
}

async function findAndGet() {
  try {
    const db = client.db("myDatabase");
    const collection = db.collection("myCollection");
    const data = await collection.findOne({});
    console.log("Data fetched successfully and decrypted data:", data);
  } catch (err) {
    console.error("Error finding and decrypting data:", err);
    throw err;
  }
}

async function main() {
  try {
    await connectToMongo();
    await insert();
    await findAndGet();
  } catch (err) {
    console.error("An unexpected error occurred:", err);
  } finally {
    await client.close();
    console.log("MongoDB connection closed");
  }
}

main();
